@@ -1,6 +1,6 @@
 /* Routines for handling XML generic OS data provided by target.
 
-   Copyright (C) 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2008-2017 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -22,7 +22,6 @@
 #include "vec.h"
 #include "xml-support.h"
 #include "osdata.h"
-#include "gdb_string.h"
 #include "ui-out.h"
 #include "gdbcmd.h"
 
@@ -45,8 +44,6 @@ osdata_parse (const char *xml)
 
 #else /* HAVE_LIBEXPAT */
 
-#include "xml-support.h"
-
 /* Internal parsing data passed to all XML callbacks.  */
 struct osdata_parsing_data
   {
@@ -61,15 +58,15 @@ osdata_start_osdata (struct gdb_xml_parser *parser,
                         const struct gdb_xml_element *element,
                         void *user_data, VEC(gdb_xml_value_s) *attributes)
 {
-  struct osdata_parsing_data *data = user_data;
+  struct osdata_parsing_data *data = (struct osdata_parsing_data *) user_data;
   char *type;
   struct osdata *osdata;
 
   if (data->osdata)
     gdb_xml_error (parser, _("Seen more than on osdata element"));
 
-  type = xml_find_attribute (attributes, "type")->value;
-  osdata = XZALLOC (struct osdata);
+  type = (char *) xml_find_attribute (attributes, "type")->value;
+  osdata = XCNEW (struct osdata);
   osdata->type = xstrdup (type);
   data->osdata = osdata;
 }
@@ -81,7 +78,7 @@ osdata_start_item (struct gdb_xml_parser *parser,
                   const struct gdb_xml_element *element,
                   void *user_data, VEC(gdb_xml_value_s) *attributes)
 {
-  struct osdata_parsing_data *data = user_data;
+  struct osdata_parsing_data *data = (struct osdata_parsing_data *) user_data;
   struct osdata_item item = { NULL };
 
   VEC_safe_push (osdata_item_s, data->osdata->items, &item);
@@ -94,8 +91,9 @@ osdata_start_column (struct gdb_xml_parser *parser,
                     const struct gdb_xml_element *element,
                     void *user_data, VEC(gdb_xml_value_s) *attributes)
 {
-  struct osdata_parsing_data *data = user_data;
-  const char *name = xml_find_attribute (attributes, "name")->value;
+  struct osdata_parsing_data *data = (struct osdata_parsing_data *) user_data;
+  const char *name
+    = (const char *) xml_find_attribute (attributes, "name")->value;
 
   data->property_name = xstrdup (name);
 }
@@ -107,7 +105,7 @@ osdata_end_column (struct gdb_xml_parser *parser,
                   const struct gdb_xml_element *element,
                   void *user_data, const char *body_text)
 {
-  struct osdata_parsing_data *data = user_data;
+  struct osdata_parsing_data *data = (struct osdata_parsing_data *) user_data;
   struct osdata *osdata = data->osdata;
   struct osdata_item *item = VEC_last (osdata_item_s, osdata->items);
   struct osdata_column *col = VEC_safe_push (osdata_column_s,
@@ -124,7 +122,7 @@ osdata_end_column (struct gdb_xml_parser *parser,
 static void
 clear_parsing_data (void *p)
 {
-  struct osdata_parsing_data *data = p;
+  struct osdata_parsing_data *data = (struct osdata_parsing_data *) p;
 
   osdata_free (data->osdata);
   data->osdata = NULL;
@@ -232,7 +230,7 @@ osdata_free (struct osdata *osdata)
 static void
 osdata_free_cleanup (void *arg)
 {
-  struct osdata *osdata = arg;
+  struct osdata *osdata = (struct osdata *) arg;
 
   osdata_free (osdata);
 }
@@ -288,33 +286,68 @@ get_osdata_column (struct osdata_item *item, const char *name)
   return NULL;
 }
 
-static void
-info_osdata_command (char *type, int from_tty)
+void
+info_osdata (const char *type)
 {
   struct ui_out *uiout = current_uiout;
   struct osdata *osdata = NULL;
-  struct osdata_item *last;
+  struct osdata_item *last = NULL;
   struct cleanup *old_chain;
-  int ncols;
-  int nprocs;
+  int ncols = 0;
+  int nrows;
+  int col_to_skip = -1;
+
+  if (type == NULL)
+    type = "";
 
   osdata = get_osdata (type);
   old_chain = make_cleanup_osdata_free (osdata);
 
-  nprocs = VEC_length (osdata_item_s, osdata->items);
+  nrows = VEC_length (osdata_item_s, osdata->items);
 
-  if (!type && nprocs == 0)
+  if (*type == '\0' && nrows == 0)
     error (_("Available types of OS data not reported."));
+  
+  if (!VEC_empty (osdata_item_s, osdata->items))
+    {
+      last = VEC_last (osdata_item_s, osdata->items);
+      if (last->columns)
+        ncols = VEC_length (osdata_column_s, last->columns);
 
-  last = VEC_last (osdata_item_s, osdata->items);
-  if (last && last->columns)
-    ncols = VEC_length (osdata_column_s, last->columns);
-  else
-    ncols = 0;
+      /* As a special case, scan the listing of available data types
+	 for a column named "Title", and only include it with MI
+	 output; this column's normal use is for titles for interface
+	 elements like menus, and it clutters up CLI output.  */
+      if (*type == '\0' && !uiout->is_mi_like_p ())
+	{
+	  struct osdata_column *col;
+	  int ix;
 
-  make_cleanup_ui_out_table_begin_end (uiout, ncols, nprocs,
+	  for (ix = 0;
+	       VEC_iterate (osdata_column_s, last->columns, ix, col);
+	       ix++)
+	    {
+	      if (strcmp (col->name, "Title") == 0)
+		col_to_skip = ix;
+	    }
+	  /* Be sure to reduce the total column count, otherwise
+	     internal errors ensue.  */
+	  if (col_to_skip >= 0)
+	    --ncols;
+	}
+    }
+
+  make_cleanup_ui_out_table_begin_end (uiout, ncols, nrows,
 				       "OSDataTable");
 
+  /* With no columns/items, we just output an empty table, but we
+     still output the table.  This matters for MI.  */
+  if (ncols == 0)
+    {
+      do_cleanups (old_chain);
+      return;
+    }
+
   if (last && last->columns)
     {
       struct osdata_column *col;
@@ -324,13 +357,21 @@ info_osdata_command (char *type, int from_tty)
           VEC_iterate (osdata_column_s, last->columns,
                        ix, col);
           ix++)
-       ui_out_table_header (uiout, 10, ui_left,
-                            col->name, col->name);
+	{
+	  char col_name[32];
+
+	  if (ix == col_to_skip)
+	    continue;
+
+	  snprintf (col_name, 32, "col%d", ix);
+	  uiout->table_header (10, ui_left,
+			       col_name, col->name);
+        }
     }
 
-  ui_out_table_body (uiout);
+  uiout->table_body ();
 
-  if (nprocs != 0)
+  if (nrows != 0)
     {
       struct osdata_item *item;
       int ix_items;
@@ -341,29 +382,40 @@ info_osdata_command (char *type, int from_tty)
           ix_items++)
        {
          struct cleanup *old_chain;
-         struct ui_stream *stb;
          int ix_cols;
          struct osdata_column *col;
 
-         stb = ui_out_stream_new (uiout);
-         old_chain = make_cleanup_ui_out_stream_delete (stb);
-         make_cleanup_ui_out_tuple_begin_end (uiout, "item");
+         old_chain = make_cleanup_ui_out_tuple_begin_end (uiout, "item");
 
          for (ix_cols = 0;
               VEC_iterate (osdata_column_s, item->columns,
                            ix_cols, col);
               ix_cols++)
-           ui_out_field_string (uiout, col->name, col->value);
+	   {
+	     char col_name[32];
 
+	     if (ix_cols == col_to_skip)
+	       continue;
+
+	     snprintf (col_name, 32, "col%d", ix_cols);
+	     uiout->field_string (col_name, col->value);
+	   }
+	 
          do_cleanups (old_chain);
 
-         ui_out_text (uiout, "\n");
+         uiout->text ("\n");
        }
     }
 
   do_cleanups (old_chain);
 }
 
+static void
+info_osdata_command (char *arg, int from_tty)
+{
+  info_osdata (arg);
+}
+
 extern initialize_file_ftype _initialize_osdata; /* -Wmissing-prototypes */
 
 void
