@@ -1,6 +1,6 @@
 /* Parser definitions for GDB.
 
-   Copyright (C) 1986-2014 Free Software Foundation, Inc.
+   Copyright (C) 1986-2017 Free Software Foundation, Inc.
 
    Modified from expread.y by the Department of Computer Science at the
    State University of New York at Buffalo.
@@ -67,11 +67,6 @@ extern CORE_ADDR expression_context_pc;
    we've encountered so far.  */
 extern const struct block *innermost_block;
 
-/* The block in which the most recently discovered symbol was found.
-   FIXME: Should be declared along with lookup_symbol in symtab.h; is not
-   related specifically to parsing.  */
-extern const struct block *block_found;
-
 /* Number of arguments seen so far in innermost function call.  */
 extern int arglist_len;
 
@@ -111,7 +106,7 @@ struct ttype
 struct symtoken
   {
     struct stoken stoken;
-    struct symbol *sym;
+    struct block_symbol sym;
     int is_a_field_of_this;
   };
 
@@ -119,7 +114,7 @@ struct objc_class_str
   {
     struct stoken stoken;
     struct type *type;
-    int class;
+    int theclass;
   };
 
 typedef struct type *type_ptr;
@@ -132,6 +127,7 @@ enum type_pieces
     tp_end = -1, 
     tp_pointer, 
     tp_reference, 
+    tp_rvalue_reference,
     tp_array, 
     tp_function,
     tp_function_with_arguments,
@@ -251,8 +247,6 @@ extern void type_stack_cleanup (void *arg);
 
 extern void push_typelist (VEC (type_ptr) *typelist);
 
-extern int length_of_subexp (struct expression *, int);
-
 extern int dump_subexp (struct expression *, struct ui_file *, int);
 
 extern int dump_subexp_body_standard (struct expression *, 
@@ -268,7 +262,7 @@ extern int operator_check_standard (struct expression *exp, int pos,
 				      (struct objfile *objfile, void *data),
 				    void *data);
 
-extern char *op_name_standard (enum exp_opcode);
+extern const char *op_name_standard (enum exp_opcode);
 
 extern struct type *follow_types (struct type *);
 
@@ -318,7 +312,7 @@ enum precedence
 
 struct op_print
   {
-    char *string;
+    const char *string;
     enum exp_opcode opcode;
     /* Precedence of operator.  These values are used only by comparisons.  */
     enum precedence precedence;
@@ -341,14 +335,14 @@ struct exp_descriptor
        the number of subexpressions it takes.  */
     void (*operator_length) (const struct expression*, int, int*, int *);
 
-    /* Call TYPE_FUNC and OBJFILE_FUNC for any TYPE and OBJFILE found being
-       referenced by the single operator of EXP at position POS.  Operator
-       parameters are located at positive (POS + number) offsets in EXP.
-       The functions should never be called with NULL TYPE or NULL OBJFILE.
-       Functions should get passed an arbitrary caller supplied DATA pointer.
-       If any of the functions returns non-zero value then (any other) non-zero
-       value should be immediately returned to the caller.  Otherwise zero
-       should be returned.  */
+    /* Call OBJFILE_FUNC for any objfile found being referenced by the
+       single operator of EXP at position POS.  Operator parameters are
+       located at positive (POS + number) offsets in EXP.  OBJFILE_FUNC
+       should never be called with NULL OBJFILE.  OBJFILE_FUNC should
+       get passed an arbitrary caller supplied DATA pointer.  If it
+       returns non-zero value then (any other) non-zero value should be
+       immediately returned to the caller.  Otherwise zero should be
+       returned.  */
     int (*operator_check) (struct expression *exp, int pos,
 			   int (*objfile_func) (struct objfile *objfile,
 						void *data),
@@ -358,7 +352,7 @@ struct exp_descriptor
        The returned value should never be NULL, even if EXP_OPCODE is
        an unknown opcode (a string containing an image of the numeric
        value of the opcode can be returned, for instance).  */
-    char *(*op_name) (enum exp_opcode);
+    const char *(*op_name) (enum exp_opcode);
 
     /* Dump the rest of this (prefix) expression after the operator
        itself has been printed.  See dump_subexp_body_standard in
