@@ -1,5 +1,4 @@
-/* Copyright 1999, 2000, 2004, 2007, 2008, 2009, 2010
-Free Software Foundation, Inc.
+/* Copyright 1999-2017 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -21,9 +20,6 @@ Free Software Foundation, Inc.
  *      on function calls.  Useful to test printing frames, stepping, etc.
  */
 
-#include <stdio.h>
-#include <unistd.h>
-
 int callee4 (void)
 {
   int A=1;
@@ -33,24 +29,23 @@ int callee4 (void)
   C = A + B;
   return 0;
 }
-callee3 (char *strarg)
+void callee3 (char *strarg)
 {
   callee4 ();
 }
 
-callee2 (int intarg, char *strarg)
+void callee2 (int intarg, char *strarg)
 {
   callee3 (strarg);
 }
 
-callee1 (int intarg, char *strarg, double fltarg)
+void callee1 (int intarg, char *strarg, double fltarg)
 {
   callee2 (intarg, strarg);
 }
 
 void callme (int i)
 {
-  printf ("callme\n");
 }
 
 int return_1 ()
@@ -62,7 +57,7 @@ void do_nothing (void)
 {
 }
 
-main ()
+int main ()
 {
   callee1 (2, "A string argument.", 3.5);
   callee1 (2, "A string argument.", 3.5);
