@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2012-2014 Free Software Foundation, Inc.
+   Copyright 2012-2017 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 int
 foo (int i, int j)
 {
+  asm ("foo_label: .globl foo_label");
   return 0;
 }
 
@@ -26,6 +27,7 @@ bar (int i)
 {
   int j = 2;
 
+  asm ("bar_label: .globl bar_label");
   return foo (i, j);
 }
 
