@@ -1,6 +1,6 @@
 /* Simulator for TI MSP430 and MSP430x processors.
 
-   Copyright (C) 2012-2014 Free Software Foundation, Inc.
+   Copyright (C) 2012-2017 Free Software Foundation, Inc.
    Contributed by Red Hat, Inc.
 
    This file is part of simulators.
@@ -21,11 +21,29 @@
 #ifndef _MSP430_SIM_H_
 #define _MSP430_SIM_H_
 
+typedef enum { UNSIGN_32, SIGN_32, UNSIGN_MAC_32, SIGN_MAC_32 } hwmult_type;
+typedef enum { UNSIGN_64, SIGN_64 } hw32mult_type;
+
 struct msp430_cpu_state
 {
   int regs[16];
   int cio_breakpoint;
   int cio_buffer;
+
+  hwmult_type  hwmult_type;
+  unsigned32   hwmult_op1;
+  unsigned32   hwmult_op2;
+  unsigned32   hwmult_result;
+  signed32     hwmult_signed_result;
+  unsigned32   hwmult_accumulator;
+  signed32     hwmult_signed_accumulator;
+
+  hw32mult_type  hw32mult_type;
+  unsigned64     hw32mult_op1;
+  unsigned64     hw32mult_op2;
+  unsigned64     hw32mult_result;
 };
 
+#define HWMULT(SD, FIELD) MSP430_CPU (SD)->state.FIELD
+
 #endif
